<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\Service\StoreConfigManager;
use Magento\Store\Model\StoreManagerInterface;
use MageWorx\SeoBase\Helper\Hreflangs;

class Urls
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var Mongo
     */
    private $mongo;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var Hreflangs
     */
    protected $hreflangHelper;
    /**
     * @var StoreConfigManager
     */
    protected $storeConfigManager;

    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     * @param StoreConfigManager $storeConfigManager
     * @param Hreflangs $hreflangs
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager,
        StoreConfigManager $storeConfigManager,
        Hreflangs $hreflangs
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->scopeConfig = $scopeConfig;
        $this->mongo = $mongo;
        $this->storeManager = $storeManager;
        $this->storeConfigManager = $storeConfigManager;
        $this->hreflangHelper = $hreflangs;
    }

    public function exportUrls()
    {
        $tableName = $this->resourceConnection->getTableName('url_rewrite');
        $this->mongo->dropCollection("urls", $this->scopeConfig->getValue('mongo/general/mongo_general_database'));
        $bulkData = [];

        $query = "SELECT $tableName.*,store.code as store_code FROM $tableName INNER JOIN store on store.store_id = $tableName.store_id WHERE $tableName.store_id != 0";

        $result = $this->resourceConnection->getConnection()->fetchAll($query);
        foreach ($result as $item) {
            $url = new \stdClass();

            $url->_id = $item['url_rewrite_id'];
            $url->url_id = $item['url_rewrite_id'];
            $url->type = $item['entity_type'] === 'contenttype_content' ? 'master_data' : $item['entity_type'];
            $url->entity_id = $item['entity_id'];
            $url->url = $item['request_path'];
            $url->is_redirect = $item['redirect_type'];
            $url->target_url = $item['target_path'];
            $url->store = $item['store_code'];
            $url->hreflang = $this->getHreflang($item['target_path'], $result);

            $bulkData[] = $url;
        }

        if (!empty($bulkData)) {
            try {
                $this->mongo->update(
                    '_id',
                    $bulkData,
                    'urls',
                    true,
                    $this->scopeConfig->getValue('mongo/general/mongo_general_database')
                );

                $this->mongo->ensureIndex(["url"], "urls");
            } catch (MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        } else {
            try {
                $this->mongo->flushCollection(
                    'urls',
                    $this->scopeConfig->getValue('mongo/general/mongo_general_database')
                );
            } catch (\MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    private function getHreflang($targetPath, $result)
    {
        $homeIdentifier = $this->scopeConfig->getValue('web/default/cms_home_page');
        $hrefArray = [];
        foreach ($result as $url) {
            if ($url['request_path'] === $homeIdentifier && isset($url['store_code']) && isset($url['store_id']) && $url['store_id'] != 0) {
                $storeConfig = $this->storeConfigManager->getStoreConfigs([$url['store_code']]);
                $storeLocale = $storeConfig[0]->getLocale();
                $urlPath = '';
                try {
                    $store = $this->storeManager->getStore($url['store_id']);
                    $urlPath = $store->getBaseUrl();
                } catch (NoSuchEntityException $e) {
                    echo $e->getMessage();
                }
                $hrefArray[$storeLocale] = $urlPath;
            }
            if ($url['target_path'] === $targetPath && isset($url['store_code']) && isset($url['store_id']) && $url['store_id'] != 0) {
                $storeConfig = $this->storeConfigManager->getStoreConfigs([$url['store_code']]);
                $storeLocale = $storeConfig[0]->getLocale();
                $urlPath = '';
                try {
                    $store = $this->storeManager->getStore($url['store_id']);
                    $urlPath = $store->getBaseUrl() . $url['request_path'];
                } catch (NoSuchEntityException $e) {
                    echo $e->getMessage();
                }
                if (!array_key_exists($storeLocale, $hrefArray)) {
                    $hrefArray[$storeLocale] = $urlPath;
                }
            }
        }
        return $hrefArray;
    }
}
